
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;


library UNISIM;
use UNISIM.VComponents.all;

entity DaqLSC is
	 Generic (gtx_rxpolarity : std_logic := '0';
				 gtx_txpolarity : std_logic := '0');
				 
    Port ( sys_reset : in  STD_LOGIC; -- active high reset of all logic but GTX
           sys_clk : in  STD_LOGIC;
           DRP_clk : in  STD_LOGIC;
           clk125 : in  STD_LOGIC;
			  --
           LinkWe : in  STD_LOGIC;
           LinkCtrl : in  STD_LOGIC;
           LinkData : in  STD_LOGIC_VECTOR (63 downto 0);
           srcID : in  STD_LOGIC_VECTOR (15 downto 0);
           LinkDown : out  STD_LOGIC;
           LinkFull : out  STD_LOGIC;
			  --
           ack_cnt : out  STD_LOGIC;        -- 1 ck pulse (txusrclk) indicating a received acknowledge
           pckt_cnt : out  STD_LOGIC;       -- 1 ck pulse (txusrclk) indicating a transmit packet
           retransmit_cnt : out  STD_LOGIC; -- 1 ck pulse (txusrclk) indicating a retransmit packet
           event_cnt : out  STD_LOGIC;      -- 1 ck pulse (sys_clk) indicating a sent event
			  sync_loss : out STD_LOGIC;		  -- goes to '1' (rxusrclk) when SERDES is out of synch
			  status_ce : in std_logic;        -- not implemented yet
			  status_addr : in STD_LOGIC_VECTOR (15 downto 0); -- not implemented yet
			  status_port : out STD_LOGIC_VECTOR (63 downto 0); -- first 32 bits are hard-wired
			  --
			  txusrclk_o : out STD_LOGIC;  -- reconstructed tx clock, to be used to clock sending circuitry
			  rxusrclk_o : out STD_LOGIC;  -- reconstructed rx clock, to be used to clock receiving circuitry
			  --
			  gtx_reset : in std_logic;    -- full reset of GTX only
			  gtx_refclk : in std_logic; -- iob for refclk neg
			  sfp_rxn : in std_logic;      -- sfp iobs
			  sfp_rxp : in std_logic;
			  sfp_txn : out std_logic;
			  sfp_txp : out std_logic
			 );
end DaqLSC;

architecture Behavioral of DaqLSC is

-- Local signals

signal gtx_txfsmresetdone : std_logic;
signal gtx_rxfsmresetdone : std_logic;
signal gtx_data_valid: std_logic;
signal sys_reset_bar : std_logic;
signal txusrclk, txusrclk2 : std_logic;
signal txdll_locked, rxdll_locked, dll_reset, rxdll_reset, txdll_reset, txplllkdet, rxplllkdet: std_logic;
signal txclkfromserdes, rxclkfromserdes, rxclkfromserdes_bufg : std_logic;
signal serdes_in_sync : std_logic;
signal txdata, rxdata : std_logic_vector(31 downto 0);
signal rxcharisk, txcharisk, rxchariscomma, gtx_rxnotintable : std_logic_vector(3 downto 0);
signal rxlossofsync : std_logic_vector(1 downto 0);
signal rxbufstatus : std_logic_vector(2 downto 0);
signal rxbyteisaligned, rxenrealign, rxbyterealign, rxcommadet : std_logic;
--signal gtxrefclk, drp_clk   : std_logic;
signal gtx_cpllfbclklost, gtx_cplllock, gtx_cpllrefclklost, gtx_rxresetdone, gtx_txresetdone : std_logic;
signal gtx_qplllock, gtx_qpllrefclklost, gtx_rxcdrlock, gtx_rxcdrlock_is_stable : std_logic;
signal stable_count : std_logic_vector(15 downto 0) ;
signal data_valid_cnt : std_logic_vector(8 downto 0) ;




	COMPONENT serdes4_GT_USRCLK_SOURCE
	PORT(
		Q2_CLK0_GTREFCLK_PAD_N_IN : IN std_logic;
		Q2_CLK0_GTREFCLK_PAD_P_IN : IN std_logic;
		GT0_TXOUTCLK_IN : IN std_logic;
		DRPCLK_IN : IN std_logic;          
		Q2_CLK0_GTREFCLK_OUT : OUT std_logic;
		GT0_TXUSRCLK_OUT : OUT std_logic;
		GT0_TXUSRCLK2_OUT : OUT std_logic;
		GT0_RXUSRCLK_OUT : OUT std_logic;
		GT0_RXUSRCLK2_OUT : OUT std_logic;
		DRPCLK_OUT : OUT std_logic
		);
	END COMPONENT;

COMPONENT SLINK_opt
	PORT(
		reset : IN std_logic;
		SYS_CLK : IN std_logic;
		LINKWe : IN std_logic;
		LINKCtrl : IN std_logic;
		LINKData : IN std_logic_vector(63 downto 0);
		src_ID : IN std_logic_vector(15 downto 0);
		inject_err : IN std_logic_vector(17 downto 0);
		read_CE : IN std_logic;
		Addr : IN std_logic_vector(15 downto 0);
		clock : IN std_logic;
		serdes_init : IN std_logic;
		clock_r : IN std_logic;
		SD_Data_i : IN std_logic_vector(31 downto 0);
		SD_Kb_i : IN std_logic_vector(3 downto 0);          
		status_data : OUT std_logic_vector(63 downto 0);
		LINKDown : OUT std_logic;
		LINK_LFF : OUT std_logic;
		SD_Data_o : OUT std_logic_vector(31 downto 0);
		SD_Kb_o : OUT std_logic_vector(3 downto 0);
		ack_cnt : OUT std_logic;
		pckt_cnt : OUT std_logic;
		retransmit : OUT std_logic;
		cnt_evt : OUT std_logic
		);
END COMPONENT;

component serdes4_init 
generic
(
    -- Simulation attributes
    EXAMPLE_SIM_GTRESET_SPEEDUP    : string    := "FALSE";    -- Set to 1 to speed up sim reset
    EXAMPLE_SIMULATION             : integer   := 0;          -- Set to 1 for simulation
    STABLE_CLOCK_PERIOD            : integer   := 20;    --Period of the stable clock driving this state-machine, unit is [ns]
    EXAMPLE_USE_CHIPSCOPE          : integer   := 0           -- Set to 1 to use Chipscope to drive resets

);
port
(
    SYSCLK_IN                               : in   std_logic;
    SOFT_RESET_IN                           : in   std_logic;
    DONT_RESET_ON_DATA_ERROR_IN             : in   std_logic;
    GT0_TX_FSM_RESET_DONE_OUT               : out  std_logic;
    GT0_RX_FSM_RESET_DONE_OUT               : out  std_logic;
    GT0_DATA_VALID_IN                       : in   std_logic;

    --_________________________________________________________________________
    --GT0  (X1Y12)
    --____________________________CHANNEL PORTS________________________________
    --------------------------------- CPLL Ports -------------------------------
    GT0_CPLLFBCLKLOST_OUT                   : out  std_logic;
    GT0_CPLLLOCK_OUT                        : out  std_logic;
    GT0_CPLLLOCKDETCLK_IN                   : in   std_logic;
    GT0_CPLLRESET_IN                        : in   std_logic;
    -------------------------- Channel - Clocking Ports ------------------------
    GT0_GTREFCLK0_IN                        : in   std_logic;
    ---------------------------- Channel - DRP Ports  --------------------------
    GT0_DRPADDR_IN                          : in   std_logic_vector(8 downto 0);
    GT0_DRPCLK_IN                           : in   std_logic;
    GT0_DRPDI_IN                            : in   std_logic_vector(15 downto 0);
    GT0_DRPDO_OUT                           : out  std_logic_vector(15 downto 0);
    GT0_DRPEN_IN                            : in   std_logic;
    GT0_DRPRDY_OUT                          : out  std_logic;
    GT0_DRPWE_IN                            : in   std_logic;
    ------------------------------- Loopback Ports -----------------------------
    GT0_LOOPBACK_IN                         : in   std_logic_vector(2 downto 0);
    ------------------------------ Power-Down Ports ----------------------------
    GT0_RXPD_IN                             : in   std_logic_vector(1 downto 0);
    GT0_TXPD_IN                             : in   std_logic_vector(1 downto 0);
    --------------------- RX Initialization and Reset Ports --------------------
    GT0_RXUSERRDY_IN                        : in   std_logic;
    -------------------------- RX Margin Analysis Ports ------------------------
    GT0_EYESCANDATAERROR_OUT                : out  std_logic;
    ------------------------- Receive Ports - CDR Ports ------------------------
    GT0_RXCDRLOCK_OUT                       : out  std_logic;
    ------------------- Receive Ports - Clock Correction Ports -----------------
    GT0_RXCLKCORCNT_OUT                     : out  std_logic_vector(1 downto 0);
    ------------------ Receive Ports - FPGA RX Interface Ports -----------------
    GT0_RXUSRCLK_IN                         : in   std_logic;
    GT0_RXUSRCLK2_IN                        : in   std_logic;
    ------------------ Receive Ports - FPGA RX interface Ports -----------------
    GT0_RXDATA_OUT                          : out  std_logic_vector(31 downto 0);
    ------------------ Receive Ports - RX 8B/10B Decoder Ports -----------------
    GT0_RXDISPERR_OUT                       : out  std_logic_vector(3 downto 0);
    GT0_RXNOTINTABLE_OUT                    : out  std_logic_vector(3 downto 0);
    --------------------------- Receive Ports - RX AFE -------------------------
    GT0_GTXRXP_IN                           : in   std_logic;
    ------------------------ Receive Ports - RX AFE Ports ----------------------
    GT0_GTXRXN_IN                           : in   std_logic;
    ------------------- Receive Ports - RX Buffer Bypass Ports -----------------
    GT0_RXBUFSTATUS_OUT                     : out  std_logic_vector(2 downto 0);
    -------------- Receive Ports - RX Byte and Word Alignment Ports ------------
    GT0_RXBYTEISALIGNED_OUT                 : out  std_logic;
    GT0_RXBYTEREALIGN_OUT                   : out  std_logic;
    GT0_RXCOMMADET_OUT                      : out  std_logic;
    GT0_RXMCOMMAALIGNEN_IN                  : in   std_logic;
    GT0_RXPCOMMAALIGNEN_IN                  : in   std_logic;
    ------------- Receive Ports - RX Initialization and Reset Ports ------------
    GT0_GTRXRESET_IN                        : in   std_logic;
    GT0_RXPMARESET_IN                       : in   std_logic;
    ----------------- Receive Ports - RX Polarity Control Ports ----------------
    GT0_RXPOLARITY_IN                       : in   std_logic;
    ------------------- Receive Ports - RX8B/10B Decoder Ports -----------------
    GT0_RXCHARISCOMMA_OUT                   : out  std_logic_vector(3 downto 0);
    GT0_RXCHARISK_OUT                       : out  std_logic_vector(3 downto 0);
    -------------- Receive Ports -RX Initialization and Reset Ports ------------
    GT0_RXRESETDONE_OUT                     : out  std_logic;
    --------------------- TX Initialization and Reset Ports --------------------
    GT0_GTTXRESET_IN                        : in   std_logic;
    GT0_TXUSERRDY_IN                        : in   std_logic;
    ------------------ Transmit Ports - FPGA TX Interface Ports ----------------
    GT0_TXUSRCLK_IN                         : in   std_logic;
    GT0_TXUSRCLK2_IN                        : in   std_logic;
    ------------------ Transmit Ports - TX Data Path interface -----------------
    GT0_TXDATA_IN                           : in   std_logic_vector(31 downto 0);
    ---------------- Transmit Ports - TX Driver and OOB signaling --------------
    GT0_GTXTXN_OUT                          : out  std_logic;
    GT0_GTXTXP_OUT                          : out  std_logic;
    ----------- Transmit Ports - TX Fabric Clock Output Control Ports ----------
    GT0_TXOUTCLK_OUT                        : out  std_logic;
    GT0_TXOUTCLKFABRIC_OUT                  : out  std_logic;
    GT0_TXOUTCLKPCS_OUT                     : out  std_logic;
    --------------------- Transmit Ports - TX Gearbox Ports --------------------
    GT0_TXCHARISK_IN                        : in   std_logic_vector(3 downto 0);
    ------------- Transmit Ports - TX Initialization and Reset Ports -----------
    GT0_TXRESETDONE_OUT                     : out  std_logic;
    ----------------- Transmit Ports - TX Polarity Control Ports ---------------
    GT0_TXPOLARITY_IN                       : in   std_logic;
   

    --____________________________COMMON PORTS________________________________
    ---------------------- Common Block  - Ref Clock Ports ---------------------
    GT0_GTREFCLK0_COMMON_IN                 : in   std_logic;
    ------------------------- Common Block - QPLL Ports ------------------------
    GT0_QPLLLOCK_OUT                        : out  std_logic;
    GT0_QPLLLOCKDETCLK_IN                   : in   std_logic;
    GT0_QPLLRESET_IN                        : in   std_logic


);
end component;



begin





--Inst_serdes4_GT_USRCLK_SOURCE: serdes4_GT_USRCLK_SOURCE
--PORT MAP(
--
--		Q2_CLK0_GTREFCLK_PAD_N_IN => gtx_refclk_n,
--		Q2_CLK0_GTREFCLK_PAD_P_IN => gtx_refclk_p,
--		Q2_CLK0_GTREFCLK_OUT => gtxrefclk,
--		
--		GT0_TXUSRCLK_OUT => txusrclk,
--		GT0_TXUSRCLK2_OUT => txusrclk2,
--		GT0_TXOUTCLK_IN => TXCLKfromSERDES,
--		GT0_RXUSRCLK_OUT => open,
--		GT0_RXUSRCLK2_OUT => open,
--		DRPCLK_IN => sys_clk,
--		DRPCLK_OUT => drp_clk 
--);
txusrclk <= clk125;
txusrclk2 <= clk125;

Inst_SLINK_opt: SLINK_opt
PORT MAP(

-- FROM FED logic	
		reset => sys_reset_bar, -- needs an active low reset
		SYS_CLK => sys_clk,
		
		
-- DATA interface from FED

		
		LINKWe => LinkWe,
		LINKCtrl => LinkCtrl,
		LINKData => LinkData,
		src_ID => srcID,
		inject_err => (others =>'0'),
		read_CE => '0',
		Addr => x"0000",
--		status_data => ,
		LINKDown => LinkDown,
		LINK_LFF => LinkFull,
		
-- SERDES interface		
		
		clock => txusrclk2,         		-- clk tx from SERDES
		serdes_init => serdes_in_sync, 	-- status that comes back from GTX
		SD_Data_o => TXDATA, 			-- data sent to serdes (32 bit)
		SD_Kb_o => TXCHARISK,         -- control K associated to SD_Data_o (4 bits)
		clock_r => txusrclk2,         -- reconstructed clock from SERDES
		SD_Data_i => RXDATA,          -- return data from SERDES 32 bit
		SD_Kb_i => RXCHARISK,         -- return control K associated to SD_Data_i (4 bits)
		
-- Status for user logic (to be removed later)

		ack_cnt => ack_cnt,    -- pulse indicating a received acknoledge
		pckt_cnt => pckt_cnt,    	-- pulse indicating a transmit packet
		retransmit => retransmit_cnt,	-- pulse indicating a retransmit packet
		cnt_evt => event_cnt 				-- pulse indicating a sent event
);
	
    serdes4_init_i : serdes4_init
    port map
    (
        SYSCLK_IN                       =>      drp_clk,
        SOFT_RESET_IN                   =>      '0',
        DONT_RESET_ON_DATA_ERROR_IN     =>      '0',
        GT0_TX_FSM_RESET_DONE_OUT       =>      gtx_txfsmresetdone,
        GT0_RX_FSM_RESET_DONE_OUT       =>      gtx_rxfsmresetdone,
        GT0_DATA_VALID_IN               =>      gtx_data_valid,

  
 
 
 
        --_____________________________________________________________________
        --_____________________________________________________________________
        --GT0  (X1Y12)

        --------------------------------- CPLL Ports -------------------------------
        GT0_CPLLFBCLKLOST_OUT           =>      gtx_cpllfbclklost,
        GT0_CPLLLOCK_OUT                =>      gtx_cplllock,
        GT0_CPLLLOCKDETCLK_IN           =>      drp_clk,
        GT0_CPLLRESET_IN                =>      '0',
        -------------------------- Channel - Clocking Ports ------------------------
        GT0_GTREFCLK0_IN                =>      gtx_refclk,
        ---------------------------- Channel - DRP Ports  --------------------------
        GT0_DRPADDR_IN                  =>      (others => '0'),
        GT0_DRPCLK_IN                   =>      drp_clk,
        GT0_DRPDI_IN                    =>      (others => '0'),
        GT0_DRPDO_OUT                   =>      open,
        GT0_DRPEN_IN                    =>      '0',
        GT0_DRPRDY_OUT                  =>      open,
        GT0_DRPWE_IN                    =>      '0',
        ------------------------------- Loopback Ports -----------------------------
        GT0_LOOPBACK_IN                 =>      "000",
        ------------------------------ Power-Down Ports ----------------------------
        GT0_RXPD_IN                     =>      "00",
        GT0_TXPD_IN                     =>      "00",
        --------------------- RX Initialization and Reset Ports --------------------
        GT0_RXUSERRDY_IN                =>      '0',
        -------------------------- RX Margin Analysis Ports ------------------------
        GT0_EYESCANDATAERROR_OUT        =>      open,
        ------------------------- Receive Ports - CDR Ports ------------------------
        GT0_RXCDRLOCK_OUT               =>      gtx_rxcdrlock,
        ------------------- Receive Ports - Clock Correction Ports -----------------
        GT0_RXCLKCORCNT_OUT             =>      open,
        ------------------ Receive Ports - FPGA RX Interface Ports -----------------
        GT0_RXUSRCLK_IN                 =>      txusrclk,
        GT0_RXUSRCLK2_IN                =>      txusrclk,
        ------------------ Receive Ports - FPGA RX interface Ports -----------------
        GT0_RXDATA_OUT                  =>      rxdata,
        ------------------ Receive Ports - RX 8B/10B Decoder Ports -----------------
        GT0_RXDISPERR_OUT               =>      open,
        GT0_RXNOTINTABLE_OUT            =>      gtx_rxnotintable,
        --------------------------- Receive Ports - RX AFE -------------------------
        GT0_GTXRXP_IN                   =>      sfp_rxp,
        ------------------------ Receive Ports - RX AFE Ports ----------------------
        GT0_GTXRXN_IN                   =>      sfp_rxn,
        ------------------- Receive Ports - RX Buffer Bypass Ports -----------------
        GT0_RXBUFSTATUS_OUT             =>      rxbufstatus,
        -------------- Receive Ports - RX Byte and Word Alignment Ports ------------
        GT0_RXBYTEISALIGNED_OUT         =>      rxbyteisaligned,
        GT0_RXBYTEREALIGN_OUT           =>      rxbyterealign,
        GT0_RXCOMMADET_OUT              =>      rxcommadet,
        GT0_RXMCOMMAALIGNEN_IN          =>      rxenrealign,
        GT0_RXPCOMMAALIGNEN_IN          =>      rxenrealign,
        ------------- Receive Ports - RX Initialization and Reset Ports ------------
        GT0_GTRXRESET_IN                =>      gtx_reset,
        GT0_RXPMARESET_IN               =>      '0',
        ----------------- Receive Ports - RX Polarity Control Ports ----------------
        GT0_RXPOLARITY_IN               =>      gtx_rxpolarity,
        ------------------- Receive Ports - RX8B/10B Decoder Ports -----------------
        GT0_RXCHARISCOMMA_OUT           =>      rxchariscomma,
        GT0_RXCHARISK_OUT               =>      rxcharisk,
        -------------- Receive Ports -RX Initialization and Reset Ports ------------
        GT0_RXRESETDONE_OUT             =>      gtx_rxresetdone,
        --------------------- TX Initialization and Reset Ports --------------------
        GT0_GTTXRESET_IN                =>      gtx_reset,
        GT0_TXUSERRDY_IN                =>      '0',
        ------------------ Transmit Ports - FPGA TX Interface Ports ----------------
        GT0_TXUSRCLK_IN                 =>      txusrclk,
        GT0_TXUSRCLK2_IN                =>      txusrclk,
        ------------------ Transmit Ports - TX Data Path interface -----------------
        GT0_TXDATA_IN                   =>      txdata,
        ---------------- Transmit Ports - TX Driver and OOB signaling --------------
        GT0_GTXTXN_OUT                  =>      sfp_txn,
        GT0_GTXTXP_OUT                  =>      sfp_txp,
        ----------- Transmit Ports - TX Fabric Clock Output Control Ports ----------
        GT0_TXOUTCLK_OUT                =>      open,
        GT0_TXOUTCLKFABRIC_OUT          =>      open,
        GT0_TXOUTCLKPCS_OUT             =>      open,
        --------------------- Transmit Ports - TX Gearbox Ports --------------------
        GT0_TXCHARISK_IN                =>      txcharisk,
        ------------- Transmit Ports - TX Initialization and Reset Ports -----------
        GT0_TXRESETDONE_OUT             =>      gtx_txresetdone,
        ----------------- Transmit Ports - TX Polarity Control Ports ---------------
        GT0_TXPOLARITY_IN               =>      gtx_txpolarity,


    --____________________________COMMON PORTS________________________________
        ---------------------- Common Block  - Ref Clock Ports ---------------------
        GT0_GTREFCLK0_COMMON_IN         =>      gtx_refclk,
        ------------------------- Common Block - QPLL Ports ------------------------
        GT0_QPLLLOCK_OUT                =>      gtx_qplllock,
        GT0_QPLLLOCKDETCLK_IN           =>      drp_clk,
        GT0_QPLLRESET_IN                =>      '0'

    );

txusrclk_o <= txusrclk;
rxusrclk_o <= txusrclk;
sys_reset_bar <= not(sys_reset);



process (txusrclk) 
begin
   if txusrclk='1' and txusrclk'event then
      if gtx_rxcdrlock='0' then 
         stable_count <= (others => '0');
			gtx_rxcdrlock_is_stable <='0';
      else 
         stable_count <= stable_count + 1;
      end if;
			if(gtx_rxnotintable /= x"0")then
				gtx_data_valid <= '0';
				data_valid_cnt <= (others => '0');
			else
				if(data_valid_cnt(8) = '1')then
					gtx_data_valid <= '1';
				else
					data_valid_cnt <= data_valid_cnt + 1;
				end if;
			end if;
   end if;
	
	if stable_count = x"ffff" then gtx_rxcdrlock_is_stable <= '1'; end if;
	
end process; 


serdes_in_sync <= gtx_rxfsmresetdone and gtx_txfsmresetdone and gtx_rxcdrlock_is_stable;

sync_loss <= not(serdes_in_sync);
	
rxenrealign <= not(rxbyteisaligned);

status_port(0) <= gtx_qplllock;
status_port(1) <= gtx_qpllrefclklost;
status_port(2) <= gtx_txresetdone;
status_port(3) <= gtx_rxresetdone;

status_port(4) <= gtx_rxcdrlock;
status_port(5) <= rxbyteisaligned;
status_port(6) <= rxbyterealign;
status_port(7) <= rxcommadet;
 
status_port(11 downto 8) <= rxchariscomma;
status_port(15 downto 12) <= rxcharisk;

status_port(16) <= gtx_cpllfbclklost;
status_port(17) <= gtx_cplllock;
status_port(18) <= gtx_cpllrefclklost;
status_port(19) <= gtx_rxfsmresetdone;
status_port(20) <= gtx_txfsmresetdone;
status_port(63 downto 21) <= (others => '0');







end Behavioral;

