------------------------------------------------------
-- data IN from FED
--
--  Ver 1.00
--
-- Dominique Gigi Feb 2012
------------------------------------------------------
--   
--  
-- 
--  
------------------------------------------------------
LIBRARY ieee;
--LIBRARY altera_mf;
--LIBRARY altera;
 

USE ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

--LIBRARY lpm;
--USE lpm.lpm_components.all; 
--USE altera_mf.altera_mf_components.all;
--USE altera.altera_primitives_components.all;

entity fed_itf is
generic (generator : boolean := false);
port (
	reset		: in std_logic;
	sys_clk		: in std_logic;
	
-- link data write enable Active Low
    LinkWe 		: in  STD_LOGIC;
-- link data header/trailer marker when '0'
	LinkCtrl 	: in  STD_LOGIC;
-- link data
	LinkData 	: in  STD_LOGIC_VECTOR (63 downto 0);
-- link data buffer almost full
	LinkAlmostFull : out  STD_LOGIC;
-- link down
	LinkDown 	: out  STD_LOGIC;
-- 
	src_ID 		: in  STD_LOGIC_VECTOR (15 downto 0);
-- enables error injection to test error recovery
	inject_err 	: in  STD_LOGIC_VECTOR (17 downto 0);
-- Link status data read out
	read_ce 	: in  STD_LOGIC;
	addr 		: in  STD_LOGIC_VECTOR (15 downto 0);
	status_data : out  STD_LOGIC_VECTOR (63 downto 0);
	
--  Interface for internal logic	
	clock		: in std_logic; -- clock from internal logic
	block_free	: in std_logic;	-- almost one block is free
	
	data_fed	: out std_logic_vector(63 downto 0);
	block_sz_fed: out std_logic_vector(15 downto 0);
	wr_ena		: out std_logic;
	start_evt	: out std_logic; -- this block is the first for the current event
	stop_evt	: out std_logic; -- this block is the last for the current event  -- both can be set
	end_blk_fed	: out std_logic;  -- indicate end of the packet (max 4KBytes)
		-- interface slave to read and write
	wr_cmd		: in std_logic;  
	func		: in std_logic_vector(15 downto 0);
	data_wr		: in std_logic_vector(31 downto 0);
	data_rd		: out std_logic_vector(31 downto 0);
	cnt_evt		: out std_logic			-- pulse for each event (on sys_clk)
 	);
	
end fed_itf;

architecture behavioral of fed_itf is

type fill_blk_type is (	idle,
						read_fifo,
						update_para
						);
signal fill_blk,fill_blkNext:fill_blk_type;

component FIFO_sync 
	port
		(
			aclr	: in std_logic; -- active low
			clk_w	: in std_logic;
			wen		: in std_logic;
			dataw	: in std_logic_vector(65 downto 0);
			almost_f: out std_logic;	-- active low
			clk_r	: in std_logic;
			datar	: out std_logic_vector(65 downto 0);
			ren		: in std_logic;
			empty	: out std_logic  -- active low
		);
end component;

component event_generator 
	port (
		reset 		: IN std_logic;
		low_clk		: IN std_logic; -- frequency of 50 Mhz
		PCIe_clk	: IN std_logic;
		PCIe_func	: IN std_logic_vector(15 downto 0);
		PCIe_wen	: IN std_logic;
		PCIe_dti	: IN std_logic_vector(31 downto 0);
		PCIe_dto	: out std_logic_vector(31 downto 0);
		PCIe_cs		: IN std_logic;
		evt_clk		: IN std_logic;
		wen			: OUT std_logic;
		data		: OUT std_logic_vector(63 downto 0);
		uctrl		: OUT std_logic;
		Back_p		: IN std_logic -- Back_p when '0'
		);
end component;

signal G_rst_rd		: std_logic;
signal data_reg		: std_logic_vector(63 downto 0);
signal datar		: std_logic_vector(65 downto 0);
signal start_evt_mem: std_logic;
signal stop_evt_mem	: std_logic;
signal end_frag		: std_logic;
signal finish_blk	: std_logic;
signal empt_ff		: std_logic;
--signal rd_ff		: std_logic;
signal rd_ff_reg	: std_logic;
signal del_rd_ff	: std_logic_vector(1 downto 0);
signal blk_size		: std_logic_vector(15 downto 0);
signal blk_full		: std_logic;
signal blk_full_anti: std_logic;
signal WEN_FF		: std_logic;
signal End_pckt_lgc	: std_logic;
signal last_word	: std_logic;
signal sel_test_mode: std_logic;
signal wen_tm		: std_logic;
signal data_tm		: std_logic_vector(63 downto 0);
signal uctrl_tm		: std_logic;
signal backpressure_mux		: std_logic;
signal wen_mux		: std_logic;
signal data_mux		: std_logic_vector(63 downto 0);
signal uctrl_mux	: std_logic;
signal PCIe_dto		: std_logic_vector(31 downto 0);
signal local_reg	: std_logic_vector(31 downto 0);
signal LINKDOWN_cell: std_logic;

	--***********************************************************
	--**********************  BEGIN  ****************************
	--***********************************************************
begin

-- for test mode 
process(reset,clock)
begin
	if reset = '0' then
		sel_test_mode 	<= '1';
		LINKDOWN_cell	<= '1';
	elsif rising_edge(clock) then
		if func(6) = '1' and wr_cmd = '1' then
			sel_test_mode 	<= data_wr(31);
			LINKDOWN_cell	<= data_wr(30);
		end if;
	end if;
end process;

local_reg(31) <= sel_test_mode;
local_reg(30) <= LINKDOWN_cell;
local_reg(29 downto 0) <= (others => '0');

LinkDown <= LINKDOWN_cell;

data_rd <= local_reg when func(6) = '1' else PCIe_dto;

--process(reset,clock,func,wr_cmd,data_wr,sel_test_mode,sys_clk,backpressure_mux)
--begin
	generator_inst:if generator generate
		i1:event_generator 
			port map(
				reset 		=> reset, 
				low_clk		=> clock, -- frequency of 100 Mhz
				PCIe_clk	=> clock, 
				PCIe_func	=> func, 
				PCIe_wen	=> wr_cmd, 
				PCIe_dti	=> data_wr, 
				PCIe_dto	=> PCIe_dto, 
				PCIe_cs		=> sel_test_mode, 
				evt_clk		=> sys_clk, 
				wen			=> wen_tm, 
				data		=> data_tm, 
				uctrl		=> uctrl_tm, 
				Back_p		=> backpressure_mux 
				);
--	else
--		PCIe_dto <= (others => '0');
	end generate;
--end process;
--**************************************************************

-- mux external and local adta path ****************************
wen_mux		<= wen_tm	when sel_test_mode = '1' and generator else LinkWe;
data_mux	<= data_tm	when sel_test_mode = '1' and generator else LinkData;	 
uctrl_mux	<= uctrl_tm	when sel_test_mode = '1' and generator else LinkCtrl; 
--**************************************************************
state_clk:process(reset,clock)
begin
if reset = '0' then
	fill_blk <= idle;
elsif rising_edge(clock) then
	fill_blk <= fill_blkNext;
end if;
end process;

DDR2a:process(fill_blk,empt_ff,block_free,blk_full,last_word)
begin 
fill_blkNext <= fill_blk;
Case fill_blk is
	when idle =>
		if empt_ff = '0' and block_free = '1' then
			fill_blkNext <= read_fifo;
		end if;
		
	when read_fifo =>
		if blk_full = '1' or last_word = '1' then --stop_evt_mem = '1' then
			fill_blkNext <= update_para;
		end if;
		
	when update_para =>
		 fill_blkNext <= idle;
		 
	when others =>
		fill_blkNext <= idle;
  end case;
end process;

WEN_FF		<= wen_mux;
end_frag	<= '1' when data_mux(63 downto 60) = x"A" and uctrl_mux = '0' else '0';

-- pulse to count the number of event dicover
process(reset,sys_clk)
begin
if reset = '0' then
	cnt_evt 	<= '0';
elsif rising_edge(sys_clk) then
	cnt_evt 	<= '0';
	if end_frag = '1' then
		cnt_evt <= '1';
	end if;
end if;
end process;

internal_FIFO:FIFO_sync --Show A Head ON
port map
	(
		aclr				=> reset,
		clk_w				=> sys_clk,
		wen					=> WEN_FF,
		dataw(63 downto 0)	=> data_mux,
		dataw(64)			=> uctrl_mux,
		dataw(65)			=> end_frag,
		almost_f			=> backpressure_mux,
		clk_r				=> clock,
		datar				=> datar,
		ren					=> rd_ff_reg,
		empty				=> empt_ff
	);

LinkAlmostFull <= '0' when sel_test_mode = '1' else backpressure_mux;

last_word 	<= '1' when rd_ff_reg = '1' and  datar(65) = '1' else '0';	
	
G_rst_rd	<= 	'0' when reset = '0' or empt_ff = '1' or blk_full = '1' else '1';
process(G_rst_rd,clock)
begin
if G_rst_rd = '0' then
	rd_ff_reg <= '0';
elsif rising_edge(clock) then
	rd_ff_reg <= '0';
	if fill_blk = read_fifo and last_word = '0' then	
		rd_ff_reg <= '1';
	end if;
end if;
end process;

process(reset,clock)
begin
if reset = '0'  then
	start_evt_mem 	<= '0';
	stop_evt_mem 	<= '0';
elsif rising_edge(clock) then
	-- generate signal to indicate the begining and the end of the event fro each the block
	if    datar(64) = '0' and datar(63 downto 60) = x"5" and rd_ff_reg = '1' then
		start_evt_mem 	<= '1';
	elsif last_word = '1' then
		stop_evt_mem 	<= '1';
	elsif fill_blk = update_para then --finish_blk = '1' then
		start_evt_mem 	<= '0';
		stop_evt_mem 	<= '0';
	end if;
	data_reg	 		<= datar(63 downto 0);
	del_rd_ff(1) 		<= del_rd_ff(0);
	del_rd_ff(0)		 <= rd_ff_reg;
end if;
end process;

process(reset,clock)
begin
if reset = '0' then
	blk_size <= (others => '0');
elsif rising_edge(clock) then
	if fill_blk = idle then	
		blk_size <= (others => '0');
	elsif rd_ff_reg = '1' and blk_full = '0' then
		blk_size <= blk_size + '1';
	end if;
end if;
end process;

process(reset,clock)
begin
if reset = '0' then
	blk_full <= '0';
elsif rising_edge(clock) then
	if blk_size = x"01FF"  and rd_ff_reg = '1' then --blk_size = 0x200
		blk_full <= '1';
	elsif End_pckt_lgc = '1' then	
		blk_full <= '0';
	end if;
end if;
end process;

--blk_full 	<= '1' when blk_size = x"0020" else '0'; 

block_sz_fed<= blk_size;
data_fed	<= data_reg;
wr_ena		<= del_rd_ff(0);
start_evt	<= start_evt_mem;
stop_evt	<= stop_evt_mem;
End_pckt_lgc<= '1' when fill_blk = update_para else '0';
end_blk_fed	<= End_pckt_lgc;

end behavioral;